/*
    Copyright© 2021 John Sennesael

    UsenetSearch is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    UsenetSearch is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with UsenetSearch.  If not, see <https://www.gnu.org/licenses/>.
*/

#pragma once

#include <filesystem>
#include <string>

#include "usenetsearch/Except.h"

namespace usenetsearch {

struct ConfigurationException: public UsenetSearchException
{
    ConfigurationException(int errorCode, const std::string& message):
        UsenetSearchException(errorCode, message){}

    virtual ~ConfigurationException() = default;
};

class Configuration
{

    std::string m_nntpServerHost{"127.0.0.1"};
    std::string m_nntpServerPassword{"password"};
    int m_nntpServerPort{119};
    bool m_nntpServerSSL{false};
    std::string m_nntpServerUser{"username"};
    std::filesystem::path m_databasePath{"./db"};

public:

    std::filesystem::path DatabasePath() const;
    std::string NNTPServerHost() const;
    std::string NNTPServerPassword() const;
    int NNTPServerPort() const;
    bool NNTPServerSSL() const;
    std::string NNTPServerUser() const;
    void Open(const std::string& filename);

};

} // namespace usenetsearch
